# Copyright (C) 2017-2019 H. Turgut Uyar <uyar@tekir.org>
#
# This file is part of Kırlent.
#
# Kırlent is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Kırlent is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Kırlent.  If not, see <http://www.gnu.org/licenses/>.

"""The module that contains the tasks for building output items."""

import copy
import sys
from pathlib import Path
from shutil import rmtree

from doit.cmd_base import TaskLoader
from doit.doit_cmd import DoitMain
from doit.loader import generate_tasks
from doit.tools import create_folder

from .model import ContentHierarchy
from .transform import generate_revealjs_sources


SPHINX_CONFIG = {
    "base": {
        "args": [
            "-b html",
            "-C",
            "-Dsource_suffix=.rv",
            "-Dextensions=%(ext)s",
            "-Dmaster_doc=%(base)s",
        ]
    },
    "math": {"extensions": ["sphinx.ext.mathjax"]},
    "ipython": {
        "extensions": [
            "IPython.sphinxext.ipython_console_highlighting",
            "IPython.sphinxext.ipython_directive",
        ]
    },
    "revealjs": {"extensions": ["kirlent_sphinx"], "args": ["-Dhtml_theme=kirlent"]},
}


# sigalias: TaskDesc = Dict[str, Any]


def task_create_build_folders(unit, folder):
    """Get the description of the build folder creation task.

    :sig: (kirlent.Unit, Path) -> TaskDesc
    :param unit: Unit to create the folders for.
    :param folder: Folder to create.
    :return: Task description.
    """
    return {
        "basename": "create_build_folders",
        "name": unit.slug,
        "actions": [(create_folder, [folder])],
        "uptodate": [folder.exists()],
        "targets": [folder],
    }


def task_clean_build_folders(unit, folder):
    """Get the description of the build folder cleanup task.

    :sig: (kirlent.Unit, Path) -> TaskDesc
    :param unit: Unit to remove the folders for.
    :param folder: Folder to remove.
    :return: Task description.
    """
    return {
        "basename": "clean_build_folders",
        "name": unit.slug,
        "actions": [(rmtree, [folder])],
        "uptodate": [not folder.exists()],
    }


def task_generate_revealjs_sources(unit, sources, output):
    """Get the description of the RevealJS source generation task.

    :sig: (kirlent.Unit, List[Path], Path) -> TaskDesc
    :param unit: Unit to generate the slides sources for.
    :param sources: Source files that make up this presentation.
    :param output: RevealJS file to store the result.
    :return: Task description.
    """
    return {
        "basename": "generate_revealjs_sources",
        "name": unit.slug,
        "actions": [(generate_revealjs_sources, [sources, output])],
        "file_dep": sources,
        "task_dep": ["create_build_folders:" + unit.slug],
        "targets": [output],
    }


def task_generate_revealjs_slides(unit, source, output):
    """Get the description of the RevealJS slides generation task.

    :sig: (kirlent.Unit, Path, Path) -> TaskDesc
    :param unit: Unit to generate the slides for.
    :param source: Source file of presentation.
    :param output: Slides file to store the result.
    :return: Task description.
    """
    sphinx_args = copy.copy(SPHINX_CONFIG.get("base").get("args"))

    extensions = []
    for extra in unit.sphinx_extras() + ["revealjs"]:
        section = SPHINX_CONFIG.get(extra)
        # TODO: check for missing section
        extensions.extend(section.get("extensions", []))
        sphinx_args.extend(section.get("args", []))

    sphinx_args.extend(["%(src)s", "%(out)s", "%(file)s"])

    sphinx_cmd = "sphinx-build " + " ".join(sphinx_args) % {
        "src": source.parent,
        "out": output.parent,
        "file": source,
        "base": source.stem,
        "ext": ",".join(extensions),
    }

    return {
        "basename": "generate_revealjs",
        "name": unit.slug,
        "actions": [sphinx_cmd],
        "file_dep": [source],
        "targets": [output],
    }


def task_generate_decktape_slides(unit, source, output):
    """Get the description of the Decktape slides generation task.

    :sig: (kirlent.Unit, Path, Path) -> TaskDesc
    :param unit: Unit to generate the slides for.
    :param source: Source file of the RevealJS slides.
    :param output: Slides file to store the result.
    :return: Task description.
    """
    decktape_args = ["decktape", "-s", "%(size)s", "file:///%(path)s", "%(file)s"]
    decktape_cmd = " ".join(decktape_args) % {
        "size": "1121x795",
        "path": source.absolute(),
        "file": output,
    }

    return {
        "basename": "generate_decktape",
        "name": unit.slug,
        "actions": [decktape_cmd],
        "file_dep": [source],
        "targets": [output],
    }


class TaskGenerator(TaskLoader):
    """Task loader that generates all Kırlent build tasks."""

    def __init__(self, src, out, *, deps):
        """Initialize this task generator.

        :sig: (Path, Path, Path) -> None
        :param src: Root folder for content sources.
        :param out: Root folder for generated output.
        :param deps: Path of task dependency database for incremental builds.
        """
        super().__init__()

        self.content = ContentHierarchy(src)  # sig: ContentHierarchy
        """Content hierarchy that stores the sources for this generator."""

        self.out = out  # sig: Path
        """Root folder where this generator stores the output it generates."""

        self.doit_config = {"dep_file": str(deps)}  # sig: Dict[str, Any]
        """Settings for doit."""

    def generate_tasks(self):
        """Generate the tasks for the units in this loader's folders.

        :sig: () -> Iterable[TaskDesc]
        :return: Task descriptions.
        """
        for unit in self.content.units.values():
            unit_out = self.out / unit.slug

            yield task_create_build_folders(unit, unit_out)

            yield task_clean_build_folders(unit, unit_out)

            lang = "en"

            slide_src = list(unit.sources(medium="slides", lang=lang))
            rv_src = Path(unit_out, f"slides.{lang}.rv")
            yield task_generate_revealjs_sources(unit, slide_src, rv_src)

            rv_out = Path(unit_out, "revealjs", rv_src.with_suffix(".html").name)
            yield task_generate_revealjs_slides(unit, rv_src, rv_out)

            dt_out = Path(unit_out, rv_out.with_suffix(".pdf").name)
            yield task_generate_decktape_slides(unit, rv_out, dt_out)

    def load_tasks(self, cmd, params, args):
        """Provide the task data."""
        return generate_tasks("kirlent", self.generate_tasks()), self.doit_config


class TaskManager:
    """A manager for controlling tasks."""

    def __init__(self, src, out, *, deps=None):
        """Initiliaze this task manager.

        :sig: (Path, Path, Path) -> None
        :param src: Root folder for the content sources.
        :param out: Root folder for generated output.
        :param deps: Path of task dependency database for incremental builds.
        """
        self.loader = TaskGenerator(src, out, deps=deps)  # sig: TaskGenerator
        """Loader for generating the tasks."""

        self.runner = None  # sig: Optional[DoitMain]
        """Runner that will process the tasks."""

    def run(self, task, *, pattern=None, exits=False):
        """Start a task.

        :sig: (str, Optional[str], Optional[bool]) -> int
        :param task: Name of task.
        :param pattern: Pattern for units to process.
        :param exits: Whether to exit after running the task.
        :return: Exit code of the task.
        """
        if self.runner is None:
            self.runner = DoitMain(self.loader)

        pattern_ = Path(pattern).name if pattern is not None else None
        task_ = f"{task}:{pattern_}" if pattern_ is not None else task
        exit_code = self.runner.run(["run", task_])
        if exits:
            sys.exit(exit_code)
        return exit_code



from pytest import fixture

from collections import namedtuple
from pathlib import Path
from shutil import rmtree


KIRLENT_ROOT = Path("/dev/shm/kirlent")


SrcOutPaths = namedtuple("Paths", ["src", "out"])


@fixture(scope="session")
def kirlent_env():
    """Source and build paths for the entire content."""
    src_root = Path(KIRLENT_ROOT, "content")
    out_root = Path(src_root.parent, "_build")

    rmtree(KIRLENT_ROOT, ignore_errors=True)
    KIRLENT_ROOT.mkdir(parents=True)

    src_root.mkdir()

    yield SrcOutPaths(src_root, out_root)

    rmtree(KIRLENT_ROOT)


@fixture
def unit1(kirlent_env):
    """Source and build paths of a unit with slides."""
    src, out = Path(kirlent_env.src, "unit1"), Path(kirlent_env.out, "unit1")
    rmtree(src, ignore_errors=True)
    rmtree(out, ignore_errors=True)
    src.mkdir()

    Path(src, "slides.en.rst").write_text("")

    yield SrcOutPaths(src, out)

    rmtree(src, ignore_errors=True)
    rmtree(out, ignore_errors=True)


@fixture
def unit2(kirlent_env):
    """Source and build paths of a unit with slides."""
    src, out = Path(kirlent_env.src, "unit2"), Path(kirlent_env.out, "unit2")
    rmtree(src, ignore_errors=True)
    rmtree(out, ignore_errors=True)
    src.mkdir()

    Path(src, "slides.en.rst").write_text("")

    yield SrcOutPaths(src, out)

    rmtree(src, ignore_errors=True)
    rmtree(out, ignore_errors=True)


@fixture
def unit3(kirlent_env, unit1, unit2):
    """Source and build paths of a unit with parts."""
    src, out = Path(kirlent_env.src, "unit3"), Path(kirlent_env.out, "unit3")
    rmtree(src, ignore_errors=True)
    rmtree(out, ignore_errors=True)
    src.mkdir()

    Path(src, "parts.txt").write_text("unit1\nunit2\n")

    yield SrcOutPaths(src, out)

    rmtree(src, ignore_errors=True)
    rmtree(out, ignore_errors=True)



from pytest import fixture

from pathlib import Path
from time import sleep

from kirlent.tasks import TaskManager


MTIME_DELAY = 0.1


@fixture(scope="module")
def task_manager(kirlent_env):
    """A task manager for generating and running test tasks."""
    deps = Path(kirlent_env.src.parent, "tasks.db")
    yield TaskManager(kirlent_env.src, kirlent_env.out, deps=deps)


def test_create_build_folders_should_create_non_existing_unit(task_manager, unit1):
    assert not unit1.out.exists()
    task_manager.run("create_build_folders:unit1")
    assert unit1.out.exists()


def test_create_build_folders_should_not_touch_existing_unit(task_manager, unit1):
    unit1.out.mkdir()
    mtime_orig = unit1.out.stat().st_mtime_ns
    sleep(MTIME_DELAY)
    task_manager.run("create_build_folders:unit1")
    assert unit1.out.stat().st_mtime_ns == mtime_orig


def test_create_build_folders_default_should_create_all_units(task_manager, unit1, unit2):
    assert not unit1.out.exists()
    assert not unit2.out.exists()
    task_manager.run("create_build_folders")
    assert unit1.out.exists()
    assert unit2.out.exists()


def test_clean_build_folders_should_delete_unit_with_contents(task_manager, unit1):
    unit1.out.mkdir()
    Path(unit1.out, "index.txt").touch()
    task_manager.run("clean_build_folders:unit1")
    assert not unit1.out.exists()


def test_clean_build_folders_should_not_fail_if_unit_does_not_exist(task_manager, unit1):
    assert not unit1.out.exists()
    exit_code = task_manager.run("clean_build_folders:unit1")
    assert exit_code == 0


def test_clean_build_folders_default_should_delete_all_unit_folders(task_manager, unit1, unit2):
    unit1.out.mkdir()
    unit2.out.mkdir()
    task_manager.run("clean_build_folders")
    assert not unit1.out.exists()
    assert not unit2.out.exists()


def test_generate_revealjs_sources_should_generate_file_with_extension_rv(task_manager, unit1):
    assert not unit1.out.exists()
    task_manager.run("generate_revealjs_sources:unit1")
    assert Path(unit1.out, "slides.en.rv").exists()


def test_generate_revealjs_sources_should_not_regenerate_if_sources_not_modified(
    task_manager, unit1
):
    assert not unit1.out.exists()
    task_manager.run("generate_revealjs_sources:unit1")
    mtime_orig = Path(unit1.out, "slides.en.rv").stat().st_mtime_ns
    sleep(MTIME_DELAY)
    task_manager.run("generate_revealjs_sources:unit1")
    assert Path(unit1.out, "slides.en.rv").stat().st_mtime_ns == mtime_orig


def test_generate_revealjs_sources_should_regenerate_if_sources_modified(task_manager, unit1):
    assert not unit1.out.exists()
    task_manager.run("generate_revealjs_sources:unit1")
    mtime_orig = Path(unit1.out, "slides.en.rv").stat().st_mtime_ns
    sleep(MTIME_DELAY)
    Path(unit1.src, "slides.en.rst").write_text("..")
    task_manager.run("generate_revealjs_sources:unit1")
    assert Path(unit1.out, "slides.en.rv").stat().st_mtime_ns > mtime_orig


def test_generate_revealjs_sources_should_regenerate_if_parts_file_modified(
    task_manager, unit3
):
    assert not unit3.out.exists()
    task_manager.run("generate_revealjs_sources:unit3")
    mtime_orig = Path(unit3.out, "slides.en.rv").stat().st_mtime_ns
    sleep(MTIME_DELAY)
    Path(unit3.src, "parts.txt").write_text("unit1\n")
    task_manager.run("generate_revealjs_sources:unit3")
    assert Path(unit3.out, "slides.en.rv").stat().st_mtime_ns > mtime_orig


def test_generate_revealjs_sources_should_regenerate_if_part_modified(
    task_manager, unit1, unit3
):
    assert not unit3.out.exists()
    task_manager.run("generate_revealjs_sources:unit3")
    mtime_orig = Path(unit3.out, "slides.en.rv").stat().st_mtime_ns
    sleep(MTIME_DELAY)
    Path(unit1.src, "slides.en.rst").write_text("..")
    task_manager.run("generate_revealjs_sources:unit3")
    assert Path(unit3.out, "slides.en.rv").stat().st_mtime_ns > mtime_orig



from pathlib import Path

from kirlent.utils import relative_path


def test_relative_path_target_in_same_folder_should_be_target_name():
    assert relative_path(Path("d1", "f"), Path("d1")).parts == ("f",)


def test_relative_path_target_in_child_folder_should_start_with_child():
    assert relative_path(Path("d1", "d2", "f"), Path("d1")).parts == ("d2", "f")


def test_relative_path_target_in_grandchild_folder_should_start_with_two_children():
    assert relative_path(Path("d1", "d2", "d3", "f"), Path("d1")).parts == ("d2", "d3", "f")


def test_relative_path_target_in_parent_folder_should_start_with_double_dots():
    assert relative_path(Path("d1", "f"), Path("d1", "d2")).parts == ("..", "f")


def test_relative_path_target_in_grandparent_folder_should_start_with_two_double_dots():
    assert relative_path(Path("d1", "f"), Path("d1", "d2", "d3")).parts == ("..", "..", "f")


def test_relative_path_target_in_diagonal_folder_should_go_up_and_down():
    assert relative_path(Path("d1", "d2", "f"), Path("d1", "d3")).parts == ("..", "d2", "f")



# THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT MANUALLY.

from typing import List, Optional

from argparse import ArgumentParser
from pathlib import Path


DEFAULT_SRC_ROOT = ...  # type: Path

def make_parser(*, prog: str) -> ArgumentParser: ...

def main(argv: Optional[List[str]] = ...) -> None: ...



# Copyright (C) 2017-2019 H. Turgut Uyar <uyar@tekir.org>
#
# This file is part of Kırlent.
#
# Kırlent is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Kırlent is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Kırlent.  If not, see <http://www.gnu.org/licenses/>.

"""The module that contains the markup transformations."""

from io import StringIO
from pathlib import Path

from .utils import relative_path


def generate_revealjs_sources(sources, target):
    """Combine a number of Kırlent sources and convert to a RevealJS file.

    :sig: (List[Path], Path) -> None
    :param sources: Paths of Kırlent source files.
    :param target: Path of target RevealJS file.
    """
    stream = StringIO()
    for source in sources:
        with source.open() as f:
            for line in f.readlines():
                if ".. image::" in line:
                    asset_path = line.split(".. image::")[1].strip()
                    full_path = Path(source.parent, asset_path)
                    rel_path = relative_path(full_path, target.parent)
                    line = line.replace(asset_path, str(rel_path))
                print(line, file=stream, end="")
        print("\n\n", file=stream)
    content = stream.getvalue()

    with target.open(mode="w") as f:
        f.write(content)



from pytest import fixture

from pathlib import Path

from kirlent.model import ContentHierarchy


@fixture(scope="module")
def kirlent_content(kirlent_env):
    """A content hierarchy that stores the content sources."""
    return ContentHierarchy(kirlent_env.src)


def test_content_hierarchy_path_should_be_absolute(kirlent_content):
    assert str(kirlent_content.root)[0] == "/"


def test_content_hierarchy_should_pick_up_units(kirlent_content, unit1):
    assert kirlent_content.units.get("unit1") is not None


def test_content_hierarchy_should_ignore_non_folders_as_units(kirlent_content):
    Path(kirlent_content.root, "unit").touch()
    assert kirlent_content.units.get("unit") is None
    Path(kirlent_content.root, "unit").unlink()


def test_content_hierarchy_should_ignore_sub_folders_as_units(kirlent_content, unit1):
    Path(unit1.src, "unit1a").mkdir()
    assert kirlent_content.units.get("unit1a") is None


def test_unit_source_path_should_be_absolute(kirlent_content, unit1):
    assert str(kirlent_content.units.get("unit1").src)[0] == "/"


def test_unit_should_have_reference_to_hierarchy(kirlent_content, unit1):
    assert kirlent_content.units.get("unit1").root == kirlent_content


def test_unit_without_parts_should_have_empty_list_of_parts(kirlent_content, unit1):
    assert kirlent_content.units.get("unit1").parts == []


def test_unit_with_parts_should_have_list_of_part_slugs(kirlent_content, unit3):
    assert kirlent_content.units.get("unit3").parts == ["unit1", "unit2"]


def test_unit_slug_should_be_same_as_folder_basename(kirlent_content, unit1):
    assert kirlent_content.units.get("unit1").slug == "unit1"


def test_unit_sources_should_be_sequence_of_absolute_paths(kirlent_content, unit1):
    unit = kirlent_content.units.get("unit1")
    assert list(unit.sources(medium="slides", lang="en")) == [
        Path(unit1.src, "slides.en.rst").absolute()
    ]


def test_unit_sources_nonexisting_language_should_be_included(kirlent_content, unit1):
    unit = kirlent_content.units.get("unit1")
    assert list(unit.sources(medium="slides", lang="tr")) == [
        Path(unit1.src, "slides.tr.rst").absolute()
    ]


def test_unit_sources_with_parts_should_be_sequence_of_part_paths(
    kirlent_content, unit1, unit2, unit3
):
    unit = kirlent_content.units.get("unit3")
    assert list(unit.sources(medium="slides", lang="en")) == [
        Path(unit1.src, "slides.en.rst").absolute(),
        Path(unit2.src, "slides.en.rst").absolute(),
    ]



# Copyright (C) 2018 H. Turgut Uyar <uyar@tekir.org>
#
# This file is part of Kırlent.
#
# Kırlent is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Kırlent is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Kırlent.  If not, see <http://www.gnu.org/licenses/>.

"""The module that contains the data representation."""

from pathlib import Path


PARTS_FILE = "parts.txt"


class ContentHierarchy:
    """A file system hierarchy for storing content."""

    def __init__(self, root):
        """Initialize this content hierarchy.

        :sig: (Path) -> None
        :param root: Root folder for the content.
        """
        self.root = root.absolute()  # sig: Path
        """Absolute path of root folder of this content hierarchy."""

    @property
    def units(self):
        """Content units in this hierarchy.

        :sig: () -> Dict[str, Unit]
        """
        return {p.name: Unit(p, root=self) for p in self.root.glob("*") if p.is_dir()}


class Unit:
    """A content unit."""

    def __init__(self, src, *, root):
        """Initialize this unit.

        :sig: (Path, ContentHierarchy) -> None
        :param src: Root folder for unit sources.
        :param root: Content hierarchy that contains this unit.
        """
        self.src = src.absolute()  # sig: Path
        """Absolute path of the root source folder of this unit."""

        self.root = root  # sig: ContentHierarchy
        """Content hierarchy that contains this unit."""

        self.parts = []  # sig: List[Unit]
        """Subunits that make up this unit."""

        parts_path = Path(self.src, PARTS_FILE)
        if parts_path.exists():
            with open(parts_path) as f:
                self.parts = [p.strip() for p in f.readlines()]

    @property
    def slug(self):
        """Short name of this unit.

        :sig: () -> str
        """
        return self.src.name

    def sources(self, *, medium, lang):
        """Get the paths of source items that make up this unit.

        :sig: (str, str) -> Sequence[Path]
        :param medium: Type of medium for which to get the sources
        :param lang: Language of content.
        :return: Source item paths.
        """
        if len(self.parts) > 0:
            for part in self.parts:
                unit = self.root.units.get(part)
                yield from unit.sources(medium=medium, lang=lang)
        else:
            if medium == "slides":
                yield Path(self.src, f"slides.{lang}.rst")
            else:
                raise NotImplementedError(
                    "Media other than slides haven't been implemented yet"
                )

    def sphinx_extras(self):
        """Get the extra features needed for Sphinx output.

        :sig: () -> List[str]
        :return: Sphinx extras.
        """
        return ["math"]



from pytest import raises

import re

from pkg_resources import get_distribution

from kirlent import cli


def test_help_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        cli.main(argv=["kirlent", "--help"])
    out, err = capsys.readouterr()
    assert out.startswith("usage: ")


def test_version_should_print_version_number_and_exit(capsys):
    version = get_distribution("kirlent").version
    with raises(SystemExit):
        cli.main(argv=["kirlent", "--version"])
    out, err = capsys.readouterr()
    assert out == "kirlent %(ver)s\n" % {"ver": version}


# TODO: Add test for the case when src directory is given
# TODO: Add test for the case when out directory is given


def test_missing_command_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        cli.main(argv=["kirlent"])
    out, err = capsys.readouterr()
    assert re.match(r"^usage: .*required: command.*", err, re.DOTALL)


def test_invalid_command_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        cli.main(argv=["kirlent", "foo"])
    out, err = capsys.readouterr()
    assert re.match(r"^usage: .*invalid choice: 'foo'.*", err, re.DOTALL)


def test_unrecognized_argument_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        cli.main(argv=["kirlent", "--foo", "clean", "unit1"])
    out, err = capsys.readouterr()
    assert re.match(r"^usage: .*unrecognized arguments: --foo.*", err, re.DOTALL)


# TODO: Add test for rv command with unit glob
# TODO: Add test for rv command without unit glob


def test_rv_unrecognized_argument_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        cli.main(argv=["kirlent", "--foo", "rv", "unit1"])
    out, err = capsys.readouterr()
    assert re.match(r"^usage: .*unrecognized arguments: --foo.*", err, re.DOTALL)


# TODO: Add test for dt command with unit glob
# TODO: Add test for dt command without unit glob


def test_dt_unrecognized_argument_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        cli.main(argv=["kirlent", "--foo", "dt", "unit1"])
    out, err = capsys.readouterr()
    assert re.match(r"^usage: .*unrecognized arguments: --foo.*", err, re.DOTALL)


# TODO: Add test for clean command with unit glob
# TODO: Add test for clean command without unit glob


def test_clean_unrecognized_argument_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        cli.main(argv=["kirlent", "--foo", "clean", "unit1"])
    out, err = capsys.readouterr()
    assert re.match(r"^usage: .*unrecognized arguments: --foo.*", err, re.DOTALL)


# TODO: Add test for task command with valid task name


def test_task_invalid_name_should_print_error_and_exit(capsys):
    with raises(SystemExit):
        cli.main(argv=["kirlent", "task", "foo"])
    out, err = capsys.readouterr()
    assert re.match(r'^ERROR: .*invalid parameter: "foo".*', err, re.DOTALL)


def test_task_missing_name_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        cli.main(argv=["kirlent", "--foo", "task"])
    out, err = capsys.readouterr()
    assert re.match(r"^usage: .*required: name.*", err, re.DOTALL)


def test_task_unrecognized_argument_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        cli.main(argv=["kirlent", "--foo", "task", "create_build_folders"])
    out, err = capsys.readouterr()
    assert re.match(r"^usage: .*unrecognized arguments: --foo.*", err, re.DOTALL)



# THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT MANUALLY.

from typing import Any, Dict, Iterable, List, Optional

from pathlib import Path
from doit.cmd_base import TaskLoader
from doit.doit_cmd import DoitMain
from .model import ContentHierarchy

import kirlent

TaskDesc = Dict[str, Any]

def task_create_build_folders(
    unit: kirlent.Unit, folder: Path
) -> TaskDesc: ...
def task_clean_build_folders(unit: kirlent.Unit, folder: Path) -> TaskDesc: ...
def task_generate_revealjs_sources(
    unit: kirlent.Unit, sources: List[Path], output: Path
) -> TaskDesc: ...
def task_generate_revealjs_slides(
    unit: kirlent.Unit, source: Path, output: Path
) -> TaskDesc: ...
def task_generate_decktape_slides(
    unit: kirlent.Unit, source: Path, output: Path
) -> TaskDesc: ...

class TaskGenerator(TaskLoader):
    content = ...  # type: ContentHierarchy
    out = ...  # type: Path
    doit_config = ...  # type: Dict[str, Any]
    def __init__(self, src: Path, out: Path, *, deps: Path) -> None: ...
    def generate_tasks(self) -> Iterable[TaskDesc]: ...

class TaskManager:
    loader = ...  # type: TaskGenerator
    runner = ...  # type: Optional[DoitMain]
    def __init__(self, src: Path, out: Path, *, deps: Path = ...) -> None: ...
    def run(
        self,
        task: str,
        *,
        pattern: Optional[str] = ...,
        exits: Optional[bool] = ...,
    ) -> int: ...



# Copyright (C) 2017 H. Turgut Uyar <uyar@tekir.org>
#
# This file is part of Kırlent.
#
# Kırlent is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Kırlent is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Kırlent.  If not, see <http://www.gnu.org/licenses/>.

"""The module that contains various utility functions."""

from itertools import dropwhile, zip_longest
from pathlib import Path


def relative_path(src, dst):
    """Get the relative path of a file starting from a destination folder.

    :sig: (Path, Path) -> Path
    :param src: Path of source file to find the relative path for.
    :param dst: Path of destination folder to start the relative path from.
    :return: Relative path from start to source file.
    """
    parts = zip_longest(src.absolute().parts, dst.absolute().parts)
    path_diff = dropwhile(lambda ps: ps[0] == ps[1], parts)
    down_parts, up_parts = zip(*path_diff)
    up_path = Path(*[".." for p in up_parts if p is not None])
    down_path = Path(*[p for p in down_parts if p is not None])
    return up_path / down_path



# THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT MANUALLY.

from typing import Dict, List, Sequence

from pathlib import Path

class ContentHierarchy:
    root = ...  # type: Path
    def __init__(self, root: Path) -> None: ...
    @property
    def units(self) -> Dict[str, Unit]: ...

class Unit:
    src = ...  # type: Path
    root = ...  # type: ContentHierarchy
    parts = ...  # type: List[Unit]
    def __init__(self, src: Path, *, root: ContentHierarchy) -> None: ...
    @property
    def slug(self) -> str: ...
    def sources(self, *, medium: str, lang: str) -> Sequence[Path]: ...
    def sphinx_extras(self) -> List[str]: ...



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['kirlent']

package_data = \
{'': ['*']}

install_requires = \
['doit>=0.31,<0.32']

extras_require = \
{'kirlent_sphinx': ['kirlent_sphinx>=0.1,<0.2']}

entry_points = \
{'console_scripts': ['kirlent = kirlent.cli:main']}

setup_kwargs = {
    'name': 'kirlent',
    'version': '0.1.0',
    'description': 'Educational content management tools.',
    'long_description': 'Copyright (C) 2017-2019 H. Turgut Uyar <uyar@tekir.org>\n\nKırlent [#meaning]_ is a collection of utilities for managing educational\ncontent.\n\n:Repository: https://gitlab.com/tekir/kirlent\n:Documentation: https://kirlent.readthedocs.io/\n\nThe idea is to generate small units of content which can then be composed\ninto larger units. A unit can contain actual content, or it can also be\na collection that consists of other content units.\n\nKırlent uses `reStructuredText`_ as its markup language for source files.\nIt defines some extra syntax to simplify authoring.\n\nThe recommended structure all content units as folders under a main folder.\nEvery document should have the language code as a prefix to its\nfile extension::\n\n   content\n   |- expressions\n   |  |- slides.en.rst\n   |  `- slides.tr.rst\n   `- statements\n\nThe output folders for generated items will replicate this hierarchy, as in::\n\n   _build\n   |- expressions\n   |  |- slides.en.pdf\n   |  `- slides.tr.pdf\n   `- statement\n\nKırlent is built on the `doit`_ tool which makes it very easy to support\nincremental builds. That is, Kırlent will build an output again only if\nany of the relevant input files have changed.\n\n.. [#meaning]\n\n   "Kırlent" is a Turkish word which means a decorative pillow. It originates\n   from the Italian word "ghirlanda". It\'s also the name of `this beauty`_.\n\n.. _reStructuredText: http://docutils.sourceforge.net/rst.html\n.. _doit: http://pydoit.org/\n.. _this beauty: https://htuyar.tumblr.com/image/87196121163\n',
    'author': 'H. Turgut Uyar',
    'author_email': 'uyar@tekir.org',
    'url': 'https://gitlab.com/tekir/kirlent',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# Copyright (C) 2017-2018 H. Turgut Uyar <uyar@tekir.org>
#
# This file is part of Kırlent.
#
# Kırlent is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Kırlent is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Kırlent.  If not, see <http://www.gnu.org/licenses/>.

"""The module that contains the command line interface for Kırlent."""

import sys
from argparse import ArgumentParser
from pathlib import Path

from . import __version__
from .tasks import TaskManager


DEFAULT_SRC_ROOT = Path("content")  # sig: Path
"""Default root folder for content sources."""


def make_parser(*, prog):
    """Build a parser for command line arguments.

    :sig: (str) -> ArgumentParser
    :param prog: Name of program.
    :return: Generated parser.
    """
    parser = ArgumentParser(prog=prog)
    parser.add_argument("--version", action="version", version="%(prog)s " + __version__)

    parser.add_argument("--src", help="root folder for content sources")
    parser.add_argument("--out", help="root folder for generated output")

    commands = parser.add_subparsers(metavar="command", dest="command")
    commands.required = True

    unit_parser = ArgumentParser(add_help=False)
    unit_parser.add_argument("units", nargs="?", help="pattern for units to process")

    rv_parser = commands.add_parser(
        "rv", help="generate HTML slides using RevealJS", parents=[unit_parser]
    )
    rv_parser.set_defaults(
        func=lambda m, a: m.run("generate_revealjs", pattern=a.units, exits=True)
    )

    dt_parser = commands.add_parser(
        "dt", help="generate PDF slides using Decktape", parents=[unit_parser]
    )
    dt_parser.set_defaults(
        func=lambda m, a: m.run("generate_decktape", pattern=a.units, exits=True)
    )

    clean_parser = commands.add_parser(
        "clean", help="clean build folders", parents=[unit_parser]
    )
    clean_parser.set_defaults(
        func=lambda m, a: m.run("clean_build_folders", pattern=a.units, exits=True)
    )

    task_parser = commands.add_parser("task", help="run specific task")
    task_parser.add_argument("name", help="task name")
    task_parser.set_defaults(func=lambda m, a: m.run(a.name, exits=True))

    return parser


def main(argv=None):
    """Start the command line interface.

    :sig: (Optional[List[str]]) -> None
    :param argv: Command line arguments.
    """
    parser = make_parser(prog="kirlent")

    argv_ = argv if argv is not None else sys.argv
    arguments = parser.parse_args(argv_[1:])

    cache_root = Path.home() / ".cache" / "kirlent"
    cache_root.mkdir(parents=True, exist_ok=True)

    tasks_db = cache_root / "tasks.db"

    src_root = Path(arguments.src) if arguments.src is not None else DEFAULT_SRC_ROOT
    out_root = Path(arguments.out) if arguments.out is not None else src_root.parent / "_build"
    task_manager = TaskManager(src_root, out_root, deps=tasks_db)

    # run the handler for the selected command
    try:
        arguments.func(task_manager, arguments)
    except RuntimeError as e:
        print(e, file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()
